﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace VaxVoIP
{
 
    public partial class AccountRecForm : UserControl
    {
        private cVaxSIPUserAgentExCOM m_objVaxUserAgent = null;
        private static AccountRecForm m_objAccountRecForm = null;
        private Boolean m_bBtnAdvanceShow = true;

        public AccountRecForm()
        {
            m_objAccountRecForm = this;

            InitializeComponent();
            m_objVaxUserAgent = cVaxSIPUserAgentExCOM.GetOBJ();

            LoadAccountData();
        }

        public static AccountRecForm GetOBJ()
        {
            return m_objAccountRecForm;
        }
                
        private void BtnOnline_Click(object sender, RoutedEventArgs e)
        {
            if (BtnOnline.Content.ToString() == "Offline")
            {
                m_objVaxUserAgent.UnRegisterAccountREC();
                BtnOnline.Content = "Online";

                return;
            }

            if (EditAuthLogin.Text == "")
            {
                MessageBox.Show("Invalid auth Login", "VaxVoIP SDK", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }

            if (EditServerAddr.Text == "")
            {
                MessageBox.Show("Invalid SIP server domain or IP address.", "VaxVoIP SDK", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }

            cStoreAccountREC.AddAccountInfo(EditDisplayName.Text, EditUserName.Text, EditAuthLogin.Text, EditAuthPwd.Text, EditDomainRealm.Text, EditServerAddr.Text, Convert.ToInt32(EditServerPort.Text), CheckRegisterToServerREC.IsChecked.Value);

            if (!m_objVaxUserAgent.RegisterAccountREC(CheckRegisterToServerREC.IsChecked.Value, EditDisplayName.Text, EditUserName.Text, EditAuthLogin.Text, EditAuthPwd.Text, EditDomainRealm.Text, EditServerAddr.Text, Convert.ToInt32(EditServerPort.Text)))
                return;

            if (CheckRegisterToServerREC.IsChecked == true)
                AddToStatusLog("Registering to SIP Server REC");
                        
            BtnOnline.Content = "Offline";
        }

        private void BtnAdvance_Click(object sender, RoutedEventArgs e)
        {
            if (m_bBtnAdvanceShow)
            {
                LabelDisplayName.Visibility = Visibility.Visible;
                LabelUserName.Visibility = Visibility.Visible;

                LabelDomain.Visibility = Visibility.Visible;
                EditDisplayName.Visibility = Visibility.Visible;

                EditUserName.Visibility = Visibility.Visible;
                EditDomainRealm.Visibility = Visibility.Visible;

                BtnAdvance.Content = "Basic";
                m_bBtnAdvanceShow = false;

                return;
            }

            LabelDisplayName.Visibility = Visibility.Hidden;
            LabelUserName.Visibility = Visibility.Hidden;

            LabelDomain.Visibility = Visibility.Hidden;
            EditDisplayName.Visibility = Visibility.Hidden;

            EditUserName.Visibility = Visibility.Hidden;
            EditDomainRealm.Visibility = Visibility.Hidden;

            BtnAdvance.Content = "Advance";
            m_bBtnAdvanceShow = true;
        }

        /////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////

        private void BtnClearStatusLog_Click(object sender, RoutedEventArgs e)
        {
            ListBoxStatusLog.Items.Clear();
        }

        private void BtnClearCallLog_Click(object sender, RoutedEventArgs e)
        {
            ListBoxCallLog.Items.Clear();
        }

        /////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////

        private void AddToStatusLog(String sStatus)
        {
            ListBoxStatusLog.Items.Add(sStatus);
        }

        private void AddToCallLog(String sStatus)
        {
            ListBoxCallLog.Items.Add(sStatus);
        }

        /////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////

        private void LoadAccountData()
        {
            String sDisplayName = "";
            String sUserName = "";

            String sAuthLogin = "";
            String sAuthPwd = "";

            String sDomainRealm = "";
            String sServerAddr = "";

            int nServerPort = 5060;
            Boolean bRegisterREC = true;

            cStoreAccountREC.GetAccountInfo(ref sDisplayName, ref sUserName, ref sAuthLogin, ref sAuthPwd, ref sDomainRealm, ref sServerAddr, ref nServerPort, ref bRegisterREC);

            EditDisplayName.Text = sDisplayName;
            EditUserName.Text = sUserName;

            EditAuthLogin.Text = sAuthLogin;
            EditAuthPwd.Text = sAuthPwd;

            EditDomainRealm.Text = sDomainRealm;
            EditServerAddr.Text = sServerAddr;

            EditServerPort.Text = Convert.ToString(nServerPort);
            CheckRegisterToServerREC.IsChecked = bRegisterREC;
        }

        /////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////

        public void OnConnectingToRegister()
        {
            AddToStatusLog("Connecting to register.");
        }

        public void OnTryingToRegister()
        {
            AddToStatusLog("Trying to register.");
        }

        public void OnFailToRegister(int nStatusCode, string sReasonPhrase)
        {
            AddToStatusLog("Failed: " + sReasonPhrase);
        }

        public void OnSuccessToRegister()
        {
            AddToStatusLog("Success to register.");
        }

        /////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////
        
        public void OnTryingToReRegister()
        {
            AddToStatusLog("Trying to Re-Register.");
        }

        public void OnFailToReRegister(int nStatusCode, string sReasonPhrase)
        {
            AddToStatusLog("Failed: " + sReasonPhrase);
        }

        public void OnSuccessToReRegister()
        {
            AddToStatusLog("Success to Re-Register.");
        }

        /////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////

        public void OnTryingToUnRegister()
        {
            AddToStatusLog("Trying to Unregister.");
        }

        public void OnFailToUnRegister()
        {
            AddToStatusLog("Fail to unregister.");
        }

        public void OnSuccessToUnRegister()
        {
            AddToStatusLog("Unregistered successfully.");
        }

        /////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////

        public void OnDialCallStarted(int nLineNo, String sCallerName, String sCallerId, String sDialNo)
        {
            AddToCallLog("Line-" + Convert.ToString(nLineNo + 1) + ": Dialing.");
        }

        public void OnDialingCall(int nLineNo, int nStatusCode, string sReasonPhrase)
        {
            AddToCallLog("Line-" + Convert.ToString(nLineNo + 1) + ": Response: " + sReasonPhrase);
        }

        public void OnConnectedCall(int nLineNo)
        {
            AddToCallLog("Line-" + Convert.ToString(nLineNo + 1) + ": Success to connect.");
        }

        public void OnDialCallFailed(int nLineNo, int nStatusCode, string sReasonPhrase)
        {
            AddToCallLog("Line-" + Convert.ToString(nLineNo + 1) + ": Response: " + sReasonPhrase);
        }

        public void OnHungupCall(int nLineNo)
        {
            AddToCallLog("Line-" + Convert.ToString(nLineNo + 1) + ": Hang up.");
        }

        /////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////

        public void OnStatusMsg(int nLineNo, int nErrorCode, string sErrorMsg)
        {
            AddToCallLog("Line-" + Convert.ToString(nLineNo + 1) + ": " + sErrorMsg);
        }

        /////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////

        private void ScrollViewer_OnListBoxScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            if (e.OriginalSource is ScrollViewer && Math.Abs(e.ExtentHeightChange) > 0.0)
            {
                ScrollViewer hScroll = (ScrollViewer)e.OriginalSource;
                hScroll.ScrollToBottom();
            }
        }
    }
}
