﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace VaxVoIP
{
    public partial class ChatForm : UserControl
    {
        private static ChatForm m_objChatForm = null;
        List<String> m_objContactNameList = new List<String>();

        private cVaxSIPUserAgentExCOM m_objVaxUserAgent = null;
        private Boolean m_bTyping = false;
        private DispatcherTimer mTimerMsgTyping = null;
                
        public ChatForm()
        {
            m_objChatForm = this;
            m_objVaxUserAgent = cVaxSIPUserAgentExCOM.GetOBJ();

            InitializeComponent();
                        
            mTimerMsgTyping = new DispatcherTimer();
            mTimerMsgTyping.Tick += new EventHandler(OnTypingTimer);
            mTimerMsgTyping.Interval = new TimeSpan(0, 0, 2);

            LabelChatTyping.Content = "";
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        public static ChatForm GetOBJ()
        {
            return m_objChatForm;
        }
                
        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        public void LoadContactAll()
        {
            m_objContactNameList.Clear();
            ListBoxContacts.Items.Clear();

            ArrayList objList = new ArrayList();
            cStoreChat.GetContactAll(ref objList);

            for (int nCount = 0; nCount < objList.Count; nCount++)
            {
                String sContactName = (String)objList[nCount];

                ListBoxContacts.Items.Add(sContactName);
                m_objContactNameList.Add(sContactName);

                m_objVaxUserAgent.ChatAddContact(sContactName);
            }

            if (ListBoxContacts.SelectedIndex == -1 && ListBoxContacts.Items.Count != 0)
            {
                ListBoxContacts.SelectedIndex = 0;
                LoadContactMsgAll((String)objList[0]);
            }
        }

        public void OnVaxOnline()
        {
            LabelChatTyping.Content = "";
            m_bTyping = false;

            LoadContactAll();
            UpdateStatusCombo();
        }

        public void OnVaxOffline()
        {
            mTimerMsgTyping.Stop();
            ListBoxStatusLog.Items.Clear();

            m_objContactNameList.Clear();
            ListBoxContacts.Items.Clear();
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        private void UpdateStatusCombo()
        {
            ComboMyChatStatus.Items.Clear();

            ComboMyChatStatus.Items.Add("Online");
            ComboMyChatStatus.Items.Add("Offline");
            ComboMyChatStatus.Items.Add("Away");
            ComboMyChatStatus.Items.Add("On the phone");
            ComboMyChatStatus.Items.Add("Busy");

            ComboMyChatStatus.SelectedIndex = cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_OFFLINE;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        private void BtnAddContact_Click(object sender, RoutedEventArgs e)
        {
            if (EditContactName.Text.Length == 0)
                return;

            if (!m_objVaxUserAgent.ChatAddContact(EditContactName.Text))
                return;

            ListBoxContacts.Items.Add(EditContactName.Text);
            m_objContactNameList.Add(EditContactName.Text);
            cStoreChat.AddContact(EditContactName.Text);

            if (ListBoxContacts.SelectedIndex == -1 && ListBoxContacts.Items.Count != 0)
                ListBoxContacts.SelectedIndex = 0;

            EditContactName.Text = "";
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        private void BtnRemoveContact_Click(object sender, RoutedEventArgs e)
        {
            int nSelectedIndex = ListBoxContacts.SelectedIndex;

            if (nSelectedIndex == -1)
                return;
                                                
            MessageBoxResult Result = MessageBox.Show("Do you really want to remove this Contact?", "Caution", MessageBoxButton.YesNo, MessageBoxImage.Question);

            if (Result == MessageBoxResult.No)
                return;

            String sContactName = m_objContactNameList[nSelectedIndex];

            m_objVaxUserAgent.ChatRemoveContact(sContactName);

            ListBoxContacts.Items.RemoveAt(nSelectedIndex);
            m_objContactNameList.RemoveAt(nSelectedIndex);
            cStoreChat.RemoveContact(sContactName);

            if (ListBoxContacts.SelectedIndex == -1 && ListBoxContacts.Items.Count != 0)
                ListBoxContacts.SelectedIndex = 0;
        }
                
        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        private void ListBoxContacts_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int nSelectedIndex = ListBoxContacts.SelectedIndex;

            if (nSelectedIndex == -1)
                return;

            ListBoxMessages.Items.Clear();
            LoadContactMsgAll(m_objContactNameList[nSelectedIndex]);
        }

        public void LoadContactMsgAll(String sContactName)
        {
            ListBoxMessages.Items.Clear();

            ArrayList objList = new ArrayList();
            cStoreChat.GetMsgAll(sContactName, ref objList);

            for (int nCount = 0; nCount < objList.Count; nCount++)
            {
                String sMsg = (String)objList[nCount];
                ListBoxMessages.Items.Add(sMsg);
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        private void ComboMyChatStatus_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int nSelectedIndex = ComboMyChatStatus.SelectedIndex;
            if (nSelectedIndex == -1) return;

            m_objVaxUserAgent.ChatSetMyStatus(nSelectedIndex);
        }
                
        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        private void BtnSendMessage_Click(object sender, RoutedEventArgs e)
        {
            if (ListBoxContacts.SelectedIndex == -1)
                return;

            if (EditSendMessage.Text.Length == 0)
                return;

            String sContactName = m_objContactNameList[ListBoxContacts.SelectedIndex];

            if (!m_objVaxUserAgent.ChatSendMessageText(sContactName, EditSendMessage.Text))
                return;

            String sMsg = "Me: " + EditSendMessage.Text;

            ListBoxMessages.Items.Add(sMsg);
            cStoreChat.AddMsg(sContactName, sMsg);

            EditSendMessage.Text = "";
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        public void OnChatSendMsgTextSuccess(String sUserName, String sMsgText, long nUserValue)
        {
            AddToStatusLog("Send Message to " + sUserName + " Success.");
        }

        public void OnChatSendMsgTextFailed(String sUserName, int nStatusCode, String sReasonPhrase, String sMsgText, long nUserValue)
        {
            AddToStatusLog("Send Message to " + sUserName + " Failed.");
        }

        private void AddToStatusLog(String sStatus)
        {
            ListBoxStatusLog.Items.Add(sStatus);
        }
                
        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        private void BtnClearMessage_Click(object sender, RoutedEventArgs e)
        {
            if (ListBoxContacts.SelectedIndex == -1)
                return;

            ListBoxMessages.Items.Clear();

            String sContactName = m_objContactNameList[ListBoxContacts.SelectedIndex];
            cStoreChat.RemoveMsgAll(sContactName);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        private String GetStatusText(int nStatusId)
        {
            String sStatus = "";

            switch (nStatusId)
            {
                case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_ONLINE:
                    sStatus = "Online";
                    break;

                case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_OFFLINE:
                    sStatus = "Offline";
                    break;

                case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_AWAY:
                    sStatus = "Away";
                    break;

                case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_ON_PHONE:
                    sStatus = "On the phone";
                    break;

                case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_BUSY:
                    sStatus = "Busy";
                    break;

                case cVaxSIPUserAgentCOM.VAX_CONTACT_STATUS_UNKNOWN:
                    sStatus = "Offline";
                    break;
            }

            return sStatus;
        }

        /////////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////////

        public void OnChatSubscribeSuccess(String sUserName)
        {
            AddToStatusLog("Chat Subscribe " + sUserName + " Success.");
        }

        public void OnChatSubscribeFailed(String sUserName, int nStatusCode, String sReasonPhrase)
        {
            AddToStatusLog("Chat Subscribe " + sUserName + " Failed.");
        }

        /////////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////////

        public void OnChatContactStatus(String sUserName, int nStatusId)
        {
            int nSelectedIndex = ListBoxContacts.SelectedIndex;
            
            for (int nContactCount = 0; nContactCount < m_objContactNameList.Count; nContactCount++)
            {
                String sContactName = m_objContactNameList[nContactCount];
                sContactName = sContactName.ToUpper();

                if (sContactName == sUserName.ToUpper())
                {
                    String sStatus = sUserName + " [" + GetStatusText(nStatusId) + "]";
                    ListBoxContacts.Items[nContactCount] = sStatus;

                    break;
                }
            }

            ListBoxContacts.SelectedIndex = nSelectedIndex;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        public void OnChatRecvMsgText(String sUserName, String sMsgText, Boolean bIsContact)
        {
            int nStart = sMsgText.IndexOf(">");

            String sMessage = "";

            if (nStart == -1)
            {
                sMessage = sUserName + ": " + sMsgText;
            }
            else
            {
                // Remote softphone is sending HTML based text messages

                nStart = nStart + 1;
                int nEnd = sMsgText.IndexOf("<", nStart);

                sMessage = sUserName + ": " + sMsgText.Substring(nStart, nEnd - nStart);
            }

            ListBoxMessages.Items.Add(sMessage);
            cStoreChat.AddMsg(sUserName, sMessage);
        }

        ///////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////////

        public void OnChatRecvMsgTypingStart(String sUserName)
        {
            m_bTyping = true;
            LabelChatTyping.Content = sUserName + " is typing..";

            mTimerMsgTyping.Stop();
            mTimerMsgTyping.Start();

            AdjustTypingState();
        }

        private void OnTypingTimer(object sender, EventArgs e)
        {
            AdjustTypingState();
        }

        private void AdjustTypingState()
        {
            if (!m_bTyping)
            {
                OnChatRecvMsgTypingStop();
                mTimerMsgTyping.Stop();
            }
            else
            {
                m_bTyping = false;
            }
        }

        public void OnChatRecvMsgTypingStop()
        {
            LabelChatTyping.Content = "";
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        private void EditSendMessage_KeyDown(object sender, KeyEventArgs e)
        {
            if (ListBoxContacts.SelectedIndex == -1)
                return;

            String sContactName = m_objContactNameList[ListBoxContacts.SelectedIndex];
            m_objVaxUserAgent.ChatSendMessageTyping(sContactName);
        }

        public void OnChatSendMsgTypingSuccess(String sUserName, long nUserValue)
        {
            AddToStatusLog("Send Typing to " + sUserName + " Success.");
        }

        public void OnChatSendMsgTypingFailed(String sUserName, int nStatusCode, String sReasonPhrase, long nUserValue)
        {
            AddToStatusLog("Send Typing to " + sUserName + " Failed.");
        }
                
        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        private void BtnClearStatusLog_Click(object sender, RoutedEventArgs e)
        {
            ListBoxStatusLog.Items.Clear();
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////

        private void ScrollViewer_OnListBoxScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            if (e.OriginalSource is ScrollViewer && Math.Abs(e.ExtentHeightChange) > 0.0)
            {
                ScrollViewer hScroll = (ScrollViewer)e.OriginalSource;
                hScroll.ScrollToBottom();
            }
        }

        
    }
}
