﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace VaxVoIP
{
 
    public partial class SettingForm : UserControl
    {
        private cVaxSIPUserAgentExCOM m_objVaxUserAgent = null;
        private static SettingForm m_objSettingForm = null;

        public SettingForm()
        {
            m_objSettingForm = this;
            InitializeComponent();

            m_objVaxUserAgent = cVaxSIPUserAgentExCOM.GetOBJ();
                        
            CheckEchoCancel.IsChecked = true;

            CheckG711U.IsChecked = true;
            CheckG729.IsChecked = true;
            CheckG711A.IsChecked = true;
            CheckGSM610.IsChecked = true;
            CheckiLBC.IsChecked = true;
            
            RadioForceNone.IsChecked = true;

            UpdateAudioIn();
            UpdateAudioOut();
            UpdateVideoDevices();

            UpdateListVideoQuality();
        }

        public static SettingForm GetOBJ()
        {
            return m_objSettingForm;
        }

        public void OnVaxOnline()
        {
            ActivateVideoDevice();
            
            ActivateAudioCodecs();
            ActivateVideoCodecs();

            ActivateVideoQuality();
            ActivateOtherSettings();
                        
            ActivateDigitDTMF();
        }
                
        //////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////

        private void UpdateAudioIn()
        {
            String sDeviceName;
            int nDeviceId;

            ListAudioIn.Items.Clear();
            ListAudioIn.Items.Add(new cMediaDevice(-1, "Default Device"));

            for (nDeviceId = 0; nDeviceId < m_objVaxUserAgent.GetAudioInDevTotal(); nDeviceId++)
            {
                sDeviceName = m_objVaxUserAgent.GetAudioInDevName(nDeviceId);

                ListAudioIn.Items.Add(new cMediaDevice(nDeviceId, sDeviceName));
            }

            ListAudioIn.Tag = true;

            ListAudioIn.SelectedIndex = 0;

            cMediaDevice objAudioDevice = (cMediaDevice)ListAudioIn.Items[0];

            nDeviceId = objAudioDevice.m_nDeviceId;
            m_objVaxUserAgent.SetActiveAudioInDevice(nDeviceId);
        }

        private void UpdateAudioOut()
        {
            int nDeviceId;
            String sDeviceName;

            ListAudioOut.Items.Clear();
            ListAudioOut.Items.Add(new cMediaDevice(-1, "Default Device"));

            for (nDeviceId = 0; nDeviceId < m_objVaxUserAgent.GetAudioOutDevTotal(); nDeviceId++)
            {
                sDeviceName = m_objVaxUserAgent.GetAudioOutDevName(nDeviceId);

                ListAudioOut.Items.Add(new cMediaDevice(nDeviceId, sDeviceName));
            }

            ListAudioOut.Tag = true;

            ListAudioOut.SelectedIndex = 0;

            cMediaDevice objAudioDevice = (cMediaDevice)ListAudioOut.Items[0];

            nDeviceId = objAudioDevice.m_nDeviceId;
            m_objVaxUserAgent.SetActiveAudioOutDevice(nDeviceId);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////////

        private void UpdateVideoDevices()
        {
            ListVideoDevices.Items.Clear();

            for (int nDeviceIndex = 0; nDeviceIndex < m_objVaxUserAgent.GetVideoDevTotal(); nDeviceIndex++)
            {
                String sDeviceName = m_objVaxUserAgent.GetVideoDevName(nDeviceIndex);
                ListVideoDevices.Items.Add(new cMediaDevice(nDeviceIndex, sDeviceName));
            }

            ListVideoDevices.Tag = true;

            if (ListVideoDevices.Items.Count > 0)
                ListVideoDevices.SelectedIndex = 0;
        }

        private void UpdateListVideoQuality()
        {
            ComboBoxVideoQuality.Items.Add("Low");
            ComboBoxVideoQuality.Items.Add("Standard");
            ComboBoxVideoQuality.Items.Add("Medium");
            ComboBoxVideoQuality.Items.Add("High");
            ComboBoxVideoQuality.Items.Add("Max");

            ComboBoxVideoQuality.Tag = true;
            ComboBoxVideoQuality.SelectedIndex = 2;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////////

        private void ActivateAudioCodecs()
        {
            m_objVaxUserAgent.DeselectAllAudioCodec();

            if (CheckG711U.IsChecked.Value)
                m_objVaxUserAgent.SelectAudioCodec(cVaxSIPUserAgentCOM.VAX_AUDIO_CODEC_G711U);
            else
                m_objVaxUserAgent.DeselectAudioCodec(cVaxSIPUserAgentCOM.VAX_AUDIO_CODEC_G711U);


            if (CheckGSM610.IsChecked.Value)
                m_objVaxUserAgent.SelectAudioCodec(cVaxSIPUserAgentCOM.VAX_AUDIO_CODEC_GSM610);
            else
                m_objVaxUserAgent.DeselectAudioCodec(cVaxSIPUserAgentCOM.VAX_AUDIO_CODEC_GSM610);


            if (CheckG711A.IsChecked.Value)
                m_objVaxUserAgent.SelectAudioCodec(cVaxSIPUserAgentCOM.VAX_AUDIO_CODEC_G711A);
            else
                m_objVaxUserAgent.DeselectAudioCodec(cVaxSIPUserAgentCOM.VAX_AUDIO_CODEC_G711A);


            if (CheckG729.IsChecked.Value)
                m_objVaxUserAgent.SelectAudioCodec(cVaxSIPUserAgentCOM.VAX_AUDIO_CODEC_G729);
            else
                m_objVaxUserAgent.DeselectAudioCodec(cVaxSIPUserAgentCOM.VAX_AUDIO_CODEC_G729);


            if (CheckiLBC.IsChecked.Value)
                m_objVaxUserAgent.SelectAudioCodec(cVaxSIPUserAgentCOM.VAX_AUDIO_CODEC_ILBC);
            else
                m_objVaxUserAgent.DeselectAudioCodec(cVaxSIPUserAgentCOM.VAX_AUDIO_CODEC_ILBC);
        }

        private void ActivateVideoCodecs()
        {
            m_objVaxUserAgent.DeselectAllVideoCodec();
            m_objVaxUserAgent.SelectVideoCodec(cVaxSIPUserAgentCOM.VAX_VIDEO_CODEC_VP8);
        }
                
        ////////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////////

        private void ActivateVideoQuality()
        {
            m_objVaxUserAgent.SetVideoQuality(ComboBoxVideoQuality.SelectedIndex);
        }

        private void ActivateVideoDevice()
        {
            m_objVaxUserAgent.SetActiveVideoDevice(ListVideoDevices.SelectedIndex);
        }

        private void ActivateOtherSettings()
        {
            m_objVaxUserAgent.EchoCancellation(CheckEchoCancel.IsChecked.Value);
            m_objVaxUserAgent.AutoGainMic(CheckAGC.IsChecked.Value, 8000);

            m_objVaxUserAgent.SetVolumeBoostMic(CheckMicBoost.IsChecked.Value);
        }
               
        ////////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////////
        
        private void ActivateDigitDTMF()
        {
            if (RadioForceNone.IsChecked == true)
            {
                m_objVaxUserAgent.ForceDigitDTMF(cVaxSIPUserAgentCOM.VAX_DTMF_TYPE_AUTO);
            }

            else if (RadioForceInbandDTMF.IsChecked == true)
            {
                //Enable Inband DTMF and VaxVoIP sends DTMF tones in the form of voice. Inband DTMF is not recommended
                //by IP-Telephony standards, because Inband DTMF only works if there is voice session with G711u or G711a codecs.

                m_objVaxUserAgent.ForceDigitDTMF(cVaxSIPUserAgentCOM.VAX_DTMF_TYPE_INBAND);
            }

            else if (RadioForceRfc2833DTMF.IsChecked == true)
            {
                m_objVaxUserAgent.ForceDigitDTMF(cVaxSIPUserAgentCOM.VAX_DTMF_TYPE_RFC2833);
            }

            else if (RadioForceInfoDTMF.IsChecked == true)
            {
                m_objVaxUserAgent.ForceDigitDTMF(cVaxSIPUserAgentCOM.VAX_DTMF_TYPE_SIP_INFO);
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////////

        private void ListAudioIn_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ListBox objListBox = (ListBox)sender;

            if (objListBox.Tag != null)
            {
                objListBox.Tag = null;
                return;
            }

            cMediaDevice objAudioDevice = (cMediaDevice)ListAudioIn.Items[ListAudioIn.SelectedIndex];
            m_objVaxUserAgent.SetActiveAudioInDevice(objAudioDevice.m_nDeviceId);
        }

        private void ListAudioOut_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ListBox objListBox = (ListBox)sender;

            if (objListBox.Tag != null)
            {
                objListBox.Tag = null;
                return;
            }

            cMediaDevice objAudioDevice = (cMediaDevice)ListAudioOut.Items[ListAudioOut.SelectedIndex];
            m_objVaxUserAgent.SetActiveAudioOutDevice(objAudioDevice.m_nDeviceId);
        }
                
        ////////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////////

        private void CheckGSM610_Click(object sender, RoutedEventArgs e)
        {
            ActivateAudioCodecs();
        }

        private void CheckiLBC_Click(object sender, RoutedEventArgs e)
        {
            ActivateAudioCodecs();
        }

        private void CheckG729_Click(object sender, RoutedEventArgs e)
        {
            ActivateAudioCodecs();
        }

        private void CheckG711U_Click(object sender, RoutedEventArgs e)
        {
            ActivateAudioCodecs();
        }

        private void CheckG711A_Click(object sender, RoutedEventArgs e)
        {
            ActivateAudioCodecs();
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////////

        private void ListVideoDevices_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ListBox objListBox = (ListBox)sender;

            if (objListBox.Tag != null)
            {
                objListBox.Tag = null;
                return;
            }
            
            ActivateVideoDevice();
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////////

        private void ComboBoxVideoQuality_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ComboBox objComboBox = (ComboBox)sender;

            if (objComboBox.Tag != null)
            {
                objComboBox.Tag = null;
                return;
            }

            ActivateVideoQuality();
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////////

        private void CheckEchoCancel_Click(object sender, RoutedEventArgs e)
        {
            ActivateOtherSettings();
        }

        private void CheckAGC_Click(object sender, RoutedEventArgs e)
        {
            ActivateOtherSettings();
        }

        private void CheckMicBoost_Click(object sender, RoutedEventArgs e)
        {
            ActivateOtherSettings();
        }
        
        ////////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////////////////////////////

        private void RadioForceNone_Click(object sender, RoutedEventArgs e)
        {
            ActivateDigitDTMF();
        }

        private void RadioForceInbandDTMF_Click(object sender, RoutedEventArgs e)
        {
            ActivateDigitDTMF();
        }

        private void RadioForceRfc2833DTMF_Click(object sender, RoutedEventArgs e)
        {
            ActivateDigitDTMF();
        }

        private void RadioForceInfoDTMF_Click(object sender, RoutedEventArgs e)
        {
            ActivateDigitDTMF();
        }

    }

    ////////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public class cMediaDevice
    {
        public int m_nDeviceId;
        public String m_sDeviceName;

        public cMediaDevice(int nDeviceId, String sDeviceName)
        {
            m_nDeviceId = nDeviceId;
            m_sDeviceName = sDeviceName;
        }

        public override String ToString()
        {
            return m_sDeviceName;
        }
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    public class cDataQoS
    {
        public int m_nQoSId;
        public String m_sQoSName;

        public cDataQoS(int nQoSId, String sQoSName)
        {
            m_nQoSId = nQoSId;
            m_sQoSName = sQoSName;
        }

        public override String ToString()
        {
            return m_sQoSName;
        }
    }
}
